package es.redsys.rest.api.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Map;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.Mac;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.DESedeKeySpec;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;

public class RestSignatureUtils {
	/** Numero de bytes para obtener cadenas multiplos de 8 */
	private static final short OCHO = 8;

	/** Constante de array de inicializacion */
	private static final byte [] IV = {0, 0, 0, 0, 0, 0, 0, 0};

	public static String createMerchantSignatureHostToHost(final String claveComercio, final String datosEntrada) throws UnsupportedEncodingException, InvalidKeyException, IllegalStateException, NoSuchAlgorithmException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, NoSuchPaddingException {

		byte [] clave = decodeB64(claveComercio.getBytes("UTF-8"));
		String secretKc = toHexadecimal(clave, clave.length);
		byte [] secretKo = encrypt_3DES(secretKc, getOrder(datosEntrada));

		// Se hace el MAC con la clave de la operacion "Ko" y se codifica en BASE64
		byte [] hash = mac256(datosEntrada, secretKo);
		String res = encodeB64String(hash);
		return res;
	}
	
	public static String createMerchantSignature(final String claveComercio, final String datosEntradaB64) throws UnsupportedEncodingException, InvalidKeyException, IllegalStateException, NoSuchAlgorithmException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, NoSuchPaddingException {
		return createMerchantSignature(claveComercio, datosEntradaB64, false);
	}
	
	public static String createMerchantSignature(final String claveComercio, final String datosEntradaB64, final Boolean safe) throws UnsupportedEncodingException, InvalidKeyException, IllegalStateException, NoSuchAlgorithmException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, NoSuchPaddingException {
		byte [] clave = decodeB64(claveComercio.getBytes("UTF-8"));
		String secretKc = toHexadecimal(clave, clave.length);
		byte [] secretKo = encrypt_3DES(secretKc, getOrder(datosEntradaB64, safe));

		// Se hace el MAC con la clave de la operacion "Ko" y se codifica en BASE64
		byte [] hash = mac256(datosEntradaB64, secretKo);
		
		String res = safe?encodeB64UrlSafe(hash):encodeB64String(hash);
		return res;
	}
	
	public static String createSignatureResponseHostToHost(final String claveComercio, final String datosEntrada, final String numPedido) throws UnsupportedEncodingException, InvalidKeyException, IllegalStateException, NoSuchAlgorithmException, IllegalBlockSizeException, BadPaddingException, InvalidAlgorithmParameterException, NoSuchPaddingException {

		byte [] clave = decodeB64(claveComercio.getBytes("UTF-8"));
		String secretKc = toHexadecimal(clave, clave.length);
		byte [] secretKo = encrypt_3DES(secretKc, numPedido);

		// Se hace el MAC con la clave de la operacion "Ko" y se codifica en BASE64
		byte [] hash = mac256(datosEntrada, secretKo);
		String res = encodeB64String(hash);
		return res;
	}

	private static byte [] encrypt_3DES(final String claveHex, final String datos) throws IllegalBlockSizeException, BadPaddingException, UnsupportedEncodingException, InvalidKeyException, InvalidAlgorithmParameterException, NoSuchAlgorithmException, NoSuchPaddingException {
		byte [] ciphertext = null;
		// Crea la clave
		DESedeKeySpec desKeySpec = new DESedeKeySpec(toByteArray(claveHex));
		SecretKey desKey = new SecretKeySpec(desKeySpec.getKey(), "DESede");
		// Crea un cifrador
		Cipher desCipher = Cipher.getInstance("DESede/CBC/NoPadding");

		// Inicializa el cifrador para encriptar
		desCipher.init(Cipher.ENCRYPT_MODE, desKey, new IvParameterSpec(IV));

		// Se aoaden los 0 en bytes necesarios para que sea un moltiplo de 8
		int numeroCerosNecesarios = OCHO - (datos.length() % OCHO);
		if (numeroCerosNecesarios == OCHO) {
			numeroCerosNecesarios = 0;
		}
		ByteArrayOutputStream array = new ByteArrayOutputStream();
		array.write(datos.getBytes("UTF-8"), 0, datos.length());
		for (int i = 0; i < numeroCerosNecesarios; i++) {
			array.write(0);
		}
		byte [] cleartext = array.toByteArray();
		// Encripta el texto
		ciphertext = desCipher.doFinal(cleartext);
		return ciphertext;
	}
	
	private static byte [] mac256(final String dsMerchantParameters, final byte [] secretKo) throws IllegalStateException, UnsupportedEncodingException, InvalidKeyException, NoSuchAlgorithmException {
		// Se hace el MAC con la clave de la operacion "Ko" y se codifica en BASE64
		Mac sha256HMAC = Mac.getInstance("HmacSHA256");
		SecretKeySpec secretKey = new SecretKeySpec(secretKo, "HmacSHA256");
		sha256HMAC.init(secretKey);
		byte [] hash = sha256HMAC.doFinal(dsMerchantParameters.getBytes("UTF-8"));
		return hash;
	}
	
	private static String getOrder(final String datos) {
		return getOrder(datos, false);
	}

	private static String getOrder(final String datos, final Boolean safe) {
		byte[] byteArray = safe?decodeB64UrlSafe(datos.getBytes()):decodeB64(datos.getBytes());
		String jsonDecoded = new String(byteArray);
		Map<String, String> jsonMap = RestTransformUtils.JSONToMap(jsonDecoded);
		
		if (jsonMap.get("DS_MERCHANT_ORDER") != null) {
			return jsonMap.get("DS_MERCHANT_ORDER");
		}
		if (jsonMap.get("Ds_Order") != null) {
			return jsonMap.get("Ds_Order"); 
		}
		return "";
	}

	public static String encodeB64String(final byte [] data) throws UnsupportedEncodingException {
		return new String(Base64.encodeBase64(data), "UTF-8");
	}

	public static String encodeB64UrlSafe(final byte [] data) throws UnsupportedEncodingException {
		byte [] encode = Base64.encodeBase64(data);
		for (int i = 0; i < encode.length; i++) {
			if (encode[i] == '+') {
				encode[i] = '-';
			} else if (encode[i] == '/') {
				encode[i] = '_';
			}
		}
		return new String(encode, "UTF-8");
	}
	
	public static byte [] decodeB64(final byte [] data) {
		return Base64.decodeBase64(data);
	}
	
	public static byte[] decodeB64UrlSafe(final byte[] encode) {
	    for (int i = 0; i < encode.length; i++) {
	        if (encode[i] == '-') {
	            encode[i] = '+';
	        } else if (encode[i] == '_') {
	            encode[i] = '/';
	        }
	    }
	    return Base64.decodeBase64(encode);
	}

	private static String toHexadecimal(byte [] datos, int numBytes) {
		String resultado = "";
		ByteArrayInputStream input = new ByteArrayInputStream(datos, 0, numBytes);
		String cadAux;
		int leido = input.read();
		while (leido != -1) {
			cadAux = Integer.toHexString(leido);
			if (cadAux.length() < 2)// Hay que aoadir un 0
				resultado += "0";
			resultado += cadAux;
			leido = input.read();
		}
		return resultado;
	}

	private static byte[] toByteArray(String cadena){
		if(cadena.length() % 2 != 0)
			cadena = "0"+cadena;
			
		int longitud = cadena.length()/2;
		int posicion = 0;
		String cadenaAux =null;
		ByteArrayOutputStream salida = new ByteArrayOutputStream();
		for(int i=0 ;i < longitud ;i++)
		{
			cadenaAux = cadena.substring(posicion,posicion+2);
			posicion +=2;
			salida.write((char)Integer.parseInt(cadenaAux,16));
		}
		return salida.toByteArray();
	}

}
